@@ -36,6 +36,12 @@ std::shared_ptr<chunk_data> apply_pixel_cube::read_chunk(chunkid_t id) {
 
     std::shared_ptr<chunk_data> out = std::make_shared<chunk_data>();
 
+    std::shared_ptr<chunk_data> in = _in_cube->read_chunk(id);
+    if (in->empty()) {
+        return out;
+    }
+
+
     // Parse expressions and create symbol table
     std::vector<double> values;
     values.resize(_in_cube->bands().count() + 9, NAN);
@@ -79,7 +85,6 @@ std::shared_ptr<chunk_data> apply_pixel_cube::read_chunk(chunkid_t id) {
         }
     }
 
-    std::shared_ptr<chunk_data> in = _in_cube->read_chunk(id);
     out->size({_bands.count(), in->size()[1], in->size()[2], in->size()[3]});
     out->buf(std::calloc(_bands.count() * in->size()[1] * in->size()[2] * in->size()[3], sizeof(double)));
 
