@@ -710,11 +710,33 @@ void vector_queries::zonal_statistics(std::shared_ptr<cube> cube, std::string og
     OGRSpatialReference srs_features = *(layer->GetSpatialRef());
     srs_features.AutoIdentifyEPSG();
 
+    bool in_ogr_was_transformed = false;
     if (!srs_cube.IsSame(&srs_features)) {
-        GCBS_ERROR("Data cube and input features have different SRSes");
+        // Transform features to srs of cube
+
+        CPLStringList translate_args;
+        translate_args.AddString("-f");
+        translate_args.AddString("GPKG");
+        translate_args.AddString("-t_srs");
+        translate_args.AddString(cube->st_reference()->srs().c_str());
+        GDALVectorTranslateOptions* opts = GDALVectorTranslateOptionsNew(translate_args.List(), NULL);
+        if (opts == NULL) {
+            GDALVectorTranslateOptionsFree(opts);
+            throw std::string("ERROR in vector_queries::zonal_statistics(): cannot create ogr2ogr options.");
+        }
+        // remember filename of temporary copy with transformed input features
+        ogr_dataset = filesystem::join(filesystem::get_tempdir(), utils::generate_unique_filename() + ".gpkg");
+        GDALDatasetH temp = GDALVectorTranslate(ogr_dataset.c_str(), NULL, 1, (GDALDatasetH*)&in_ogr_dataset,opts, NULL);
+        if (!temp) {
+            GDALClose(in_ogr_dataset);
+            GCBS_ERROR("Failed to transform input feature to data cube SRS");
+            throw std::string("Failed to transform input feature to data cube SRS");
+        }
         GDALClose(in_ogr_dataset);
-        // TODO: do we have to clean up more things here?
-        throw std::string("Data cube and input features have different SRSes");
+        in_ogr_dataset = (GDALDataset*)temp;
+        layer = in_ogr_dataset->GetLayerByName(ogr_layer.c_str());
+        GDALVectorTranslateOptionsFree(opts);
+        in_ogr_was_transformed = true;
     }
 
     // Helper data structure: map: chunk index -> vector of FIDs of all features intersecting the chunk
@@ -1138,6 +1160,9 @@ void vector_queries::zonal_statistics(std::shared_ptr<cube> cube, std::string og
     }
 
     GDALClose(gpkg_out);
+    if (in_ogr_was_transformed) { // if temporary copy (due to coordinate transformation needed)
+        filesystem::remove(ogr_dataset);
+    }
     prg->finalize();
 }
 
